from snips_nlu.cli.download import download
from snips_nlu.cli.generate_dataset import generate_dataset
from snips_nlu.cli.inference import parse
from snips_nlu.cli.link import link
from snips_nlu.cli.metrics import train_test_metrics, cross_val_metrics
from snips_nlu.cli.training import train
