# inspired from:
# https://python-packaging-user-guide.readthedocs.io/guides/single-sourcing-package-version/
# https://github.com/pypa/warehouse/blob/master/warehouse/__about__.py

# pylint:disable=line-too-long

__title__ = "snips_nlu"
__summary__ = "Snips Natural Language Understanding library"
__uri__ = "https://snips-nlu.readthedocs.io"
__author__ = "Clement Doumouro, Adrien Ball"
__email__ = "clement.doumouro@snips.ai, adrien.ball@snips.ai"
__license__ = "Apache License, Version 2.0"

__version__ = "0.15.1"
__model_version__ = "0.15.0"

__download_url__ = "https://github.com/snipsco/snips-nlu-language-resources/releases/download"
__compatibility__ = "https://raw.githubusercontent.com/snipsco/snips-nlu-language-resources/master/compatibility.json"
__shortcuts__ = "https://raw.githubusercontent.com/snipsco/snips-nlu-language-resources/master/shortcuts.json"

# pylint:enable=line-too-long
